/********************************************************************************
 *   dbgreader -- system debug output reader for Windows CE                     *
 *                                                                              * 
 *   Copyright (C) 2004-2007 dci (dev@cpp.in)                                   *
 *                                                                              *
 *   This program is free software; you can redistribute it and/or              *
 *   modify it under the terms of the GNU General Public License version 2      *
 *   as published by the Free Software Foundation.                              *
 *                                                                              *
 *   This program is distributed in the hope that it will be useful,            *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
 *   General Public License version 2 for more details.                         *
 *                                                                              *
 *   You should have received a copy of the GNU General Public License          *
 *   version 2 along with this library; if not, write to the Free Software      *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 ********************************************************************************
 */


#include <windows.h>
/* '#include <pkfuncs.h>' should be here */
HANDLE LoadIntChainHandler(LPCTSTR lpszFileName, LPCTSTR lpszFunctionName, BYTE bIRQ);
BOOL FreeIntChainHandler(HANDLE hInstance);
BOOL KernelLibIoControl(HANDLE hLib, DWORD dwIoControlCode, LPVOID lpInBuf, DWORD nInBufSize, LPVOID lpOutBuf, DWORD nOutBufSize, LPDWORD lpBytesReturned);

#include "dbgreader.h"


int main ( int argc, char * argv[] )
{
	/* let user to trust the server */
	HMODULE kTrust = LoadLibrary ( L"dbgserver.dll" );
	if ( kTrust ) FreeLibrary ( kTrust );
	/* */
	printf ( "Starting dbgreader\n(c) 2007 dci\n" );
	/* */
	if ( OEM_CERTIFY_TRUST == CeGetCurrentTrust() )
	{
		/* load server to kernel */
		HANDLE kDbgServer = LoadIntChainHandler ( L"dbgserver.dll", NULL, 0 );
		if ( kDbgServer )
		{
			DWORD test = 0;
			/* */
			printf ( "Server loaded at %08X\n", kDbgServer );
			/* read traced offset */
			if ( KernelLibIoControl ( kDbgServer, IOCTL_DBGSERVER_TEST, NULL, 0, &test, sizeof ( test ), NULL ) )
			{
				printf ( "Test: %08X\n", test );
			}
			else
			{
				printf ( "IOCTL_DBGSERVER_TEST failed!\n" );
			}
			/* ok? */
			if ( test )
			{
				DWORD newstate = HS_ENABLE;
				DWORD oldstate = 0;
				if ( KernelLibIoControl ( kDbgServer, IOCTL_DBGSERVER_STATE_SET, &newstate, sizeof ( newstate ), &oldstate, sizeof ( oldstate ), NULL ) )
				{
					unsigned short usLastReadMsg = 0;
					unsigned long  ulLastProc = 0xFFFFFFFF;
					int iLineBreak = 1;
					/* */
					printf ( "Right arrow aborts\n" );
					printf ( "dbgserver state %d->%d\n---------------\n", oldstate, newstate );
					/* read initial id */
					KernelLibIoControl ( kDbgServer, IOCTL_DBGSERVER_MSG_LAST, NULL, 0, &usLastReadMsg, sizeof ( usLastReadMsg ), NULL );
					/* */
					while ( 1 )
					{
						unsigned short lastmsg = 0;
						/* */
						KernelLibIoControl ( kDbgServer, IOCTL_DBGSERVER_MSG_LAST, NULL, 0, &lastmsg, sizeof ( lastmsg ), NULL );
						if ( lastmsg != usLastReadMsg )
						{
							while ( lastmsg != usLastReadMsg )
							{
								char msgbuf [ 2048 ];
								++usLastReadMsg;
								/* */
								if ( KernelLibIoControl ( kDbgServer, IOCTL_DBGSERVER_MSG_READ, &usLastReadMsg, sizeof ( usLastReadMsg ), msgbuf, sizeof ( msgbuf ), NULL ) )
								{
									unsigned long pid =  ( ( msginfo_t * ) msgbuf )->dwProcessId;
									wchar_t *     msg = ( wchar_t * ) ( msgbuf + sizeof ( msginfo_t ) );
									/* process changed? */
									if ( ulLastProc != pid )
									{
										wchar_t procpath [ MAX_PATH ], * procname;
										/* line wasn't finished */
										if ( 0 == iLineBreak ) printf ( "\n" );
										/* show process name */
										if ( pid )
										{
											if ( 0 == GetModuleFileName ( ( HMODULE ) pid, procpath, MAX_PATH ) ) wcscpy ( procpath, L"unknown" );
										}
										else
										{
											wcscpy ( procpath, L"system" );
										}
										procname = wcsrchr ( procpath, '\\' );
										printf ( "[%S]\n", procname ? procname + 1 : procpath );
										/* */
										ulLastProc = pid;
									}
									/* show message */
									printf ( "%S", msg );
									/* is msg finished? */
									msg += ( wcslen ( msg ) - 1 );
									iLineBreak = ( ( '\n' == * msg ) || ( '\r' == * msg ) ) ? 1 : 0;
								}
								else
								{
									printf ( "[IOCTL_DBGSERVER_MSG_READ failed]\n" );
								}
							}
						}
						else
						{
							Sleep ( 100 );
						}
						/* time to exit? */
						if ( GetAsyncKeyState ( VK_RIGHT ) ) break;
					}
					/* */
					if ( 0 == iLineBreak ) printf ( "\n" );
					/* */
					newstate = HS_DISABLE;
					if ( KernelLibIoControl ( kDbgServer, IOCTL_DBGSERVER_STATE_SET, &newstate, sizeof ( newstate ), &oldstate, sizeof ( oldstate ), NULL ) )
					{
						printf ( "---------------\ndbgserver state %d->%d\n", oldstate, newstate );
					}
				}
				else
				{
					printf ( "[IOCTL_DBGSERVER_STATE_SET failed]\n" );
				}
			}
			else
			{
				printf ( "Test error\n" );
			}
			/* unload lib */
			FreeIntChainHandler ( kDbgServer );
		}
		else
		{
			printf ( "Unable to load dbgserver\n" );
		}
	}
	else
	{
		printf ( "Not trusted.\n" );
	}
	/* */
	printf ( "Press left arrow to quit\n" );
	while ( 0 == GetAsyncKeyState ( VK_LEFT ) ) Sleep ( 50 );
	return 0;
}
